/*
 * Apache License
 * Version 2.0, January 2004
 * http://www.apache.org/licenses/
 *
 *    Copyright 2013 Aurelian Tutuianu
 *    Copyright 2014 Aurelian Tutuianu
 *    Copyright 2015 Aurelian Tutuianu
 *    Copyright 2016 Aurelian Tutuianu
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 *
 */

package rapaio.experiment.ml.classifier.tree.ctree;


import org.junit.jupiter.api.Test;
import rapaio.core.tools.DensityTable;
import rapaio.data.Frame;
import rapaio.data.VType;
import rapaio.data.filter.FRetainTypes;
import rapaio.datasets.Datasets;
import rapaio.experiment.ml.classifier.tree.CTree;
import rapaio.ml.eval.metric.Confusion;

import static org.junit.jupiter.api.Assertions.assertTrue;

/**
 * User: Aurelian Tutuianu <padreati@yahoo.com>
 */
public class C45ClassifierModelTest {

    @Test
    void testNominalInfoGain() {
        Frame df = Datasets.loadPlay();
        df = FRetainTypes.on(VType.NOMINAL).fapply(df);
        final String className = "class";

        CTree classifier = CTree.newC45();
        classifier.fit(df, className);
        var pred = classifier.predict(df);

        var dtWindy = DensityTable.fromLevelCounts(false, df.rvar("windy"), df.rvar("class"));
        var dtOutlook = DensityTable.fromLevelCounts(false, df.rvar("outlook"), df.rvar("class"));
        String splitCol = (dtWindy.getTools().splitByRowInfoGain() > dtOutlook.getTools().splitByRowInfoGain()) ? "windy" : "outlook";
        assertTrue(classifier.getRoot().getBestCandidate().getGroupPredicates().get(0).toString().contains(splitCol));

        classifier.printSummary();

        Confusion cm = Confusion.from(df.rvar("class"), pred.firstClasses());
        cm.printSummary();
    }

    @Test
    public void testNumericInfoGain() {
        Frame df = Datasets.loadPlay();
        df = df.mapVars("temp,humidity,class");
        final String className = "class";

        CTree classifier = CTree.newC45();
        classifier.fit(df, className);
        classifier.printSummary();

        var pred = classifier.predict(df);

        Confusion cm = Confusion.from(df.rvar("class"), pred.firstClasses());
        cm.printSummary();
    }

    @Test
    public void testAllInfoGain() {
        Frame df = Datasets.loadPlay();
        final String className = "class";

        CTree classifier = CTree.newC45().withMinCount(1);
        classifier.fit(df, className);
        classifier.printSummary();

        var pred = classifier.predict(df);

        Confusion cm = Confusion.from(df.rvar("class"), pred.firstClasses());
        cm.printSummary();
    }
}
